<?php

namespace Drupal\ai_upgrade_assistant\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\State\StateInterface;
use GuzzleHttp\ClientInterface;
use Drupal\ai_upgrade_assistant\Service\DataAnonymizer;

/**
 * Service for managing community learning and pattern sharing.
 */
class CommunityLearningService {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The data anonymizer service.
   *
   * @var \Drupal\ai_upgrade_assistant\Service\DataAnonymizer
   */
  protected $dataAnonymizer;

  /**
   * Constructs a new CommunityLearningService.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\ai_upgrade_assistant\Service\DataAnonymizer $data_anonymizer
   *   The data anonymizer service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    Connection $database,
    ClientInterface $http_client,
    LoggerChannelFactoryInterface $logger_factory,
    StateInterface $state,
    DataAnonymizer $data_anonymizer
  ) {
    $this->configFactory = $config_factory;
    $this->database = $database;
    $this->httpClient = $http_client;
    $this->loggerFactory = $logger_factory->get('ai_upgrade_assistant');
    $this->state = $state;
    $this->dataAnonymizer = $data_anonymizer;
  }

  /**
   * Stores a new upgrade pattern locally.
   *
   * @param array $pattern
   *   The upgrade pattern to store.
   *
   * @return int|null
   *   The ID of the stored pattern, or NULL if storage failed.
   */
  public function storePattern(array $pattern): ?int {
    try {
      // Store the original pattern for local reference.
      $id = $this->database->insert('ai_upgrade_patterns')
        ->fields([
          'type' => $pattern['type'],
          'module_name' => $pattern['module_name'],
          'from_version' => $pattern['from_version'],
          'to_version' => $pattern['to_version'],
          'pattern_data' => serialize($pattern),
          'context_data' => serialize($pattern['context'] ?? []),
          'success' => $pattern['success'] ?? 0,
          'sync_status' => 'pending',
          'created' => time(),
          'changed' => time(),
        ])
        ->execute();

      $this->loggerFactory->notice('Stored new upgrade pattern for @module', [
        '@module' => $pattern['module_name'],
      ]);

      return $id;
    }
    catch (\Exception $e) {
      $this->loggerFactory->error('Failed to store upgrade pattern: @error', [
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

  /**
   * Shares patterns with the central learning system.
   *
   * @return bool
   *   TRUE if patterns were shared successfully, FALSE otherwise.
   */
  public function sharePatterns(): bool {
    if (!$this->isSharingEnabled()) {
      return FALSE;
    }

    try {
      // Get pending patterns.
      $patterns = $this->database->select('ai_upgrade_patterns', 'p')
        ->fields('p')
        ->condition('sync_status', 'pending')
        ->execute()
        ->fetchAll();

      $shared = 0;
      foreach ($patterns as $pattern) {
        $data = unserialize($pattern->pattern_data);
        
        // Anonymize the pattern.
        $anonymized = $this->dataAnonymizer->anonymizePattern($data);
        
        // Validate anonymization.
        if (!$this->dataAnonymizer->validateAnonymization($anonymized)) {
          $this->loggerFactory->warning('Pattern failed anonymization validation');
          continue;
        }

        // Share with central system.
        $response = $this->httpClient->post('https://ai-upgrade-training.drupal.org/api/patterns', [
          'json' => $anonymized,
          'headers' => $this->getAuthHeaders(),
        ]);

        if ($response->getStatusCode() === 200) {
          // Update sync status.
          $this->database->update('ai_upgrade_patterns')
            ->fields([
              'sync_status' => 'synced',
              'changed' => time(),
            ])
            ->condition('id', $pattern->id)
            ->execute();
          $shared++;
        }
      }

      $this->loggerFactory->notice('Shared @count patterns with central learning system', [
        '@count' => $shared,
      ]);

      return TRUE;
    }
    catch (\Exception $e) {
      $this->loggerFactory->error('Failed to share patterns: @error', [
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
   * Checks if pattern sharing is enabled.
   *
   * @return bool
   *   TRUE if sharing is enabled, FALSE otherwise.
   */
  public function isSharingEnabled(): bool {
    return $this->configFactory
      ->get('ai_upgrade_assistant.settings')
      ->get('enable_sharing') ?? FALSE;
  }

  /**
   * Gets authentication headers for API requests.
   *
   * @return array
   *   Array of headers.
   */
  protected function getAuthHeaders(): array {
    $config = $this->configFactory->get('ai_upgrade_assistant.settings');
    return [
      'Authorization' => 'Bearer ' . $config->get('api_key'),
      'X-Site-Token' => $this->state->get('ai_upgrade_assistant.site_token'),
    ];
  }

  /**
   * Previews what data would be shared from a pattern.
   *
   * @param int $pattern_id
   *   The ID of the pattern to preview.
   *
   * @return array|null
   *   Preview data showing original and anonymized versions, or NULL if not found.
   */
  public function previewPatternSharing(int $pattern_id): ?array {
    try {
      $pattern = $this->database->select('ai_upgrade_patterns', 'p')
        ->fields('p')
        ->condition('id', $pattern_id)
        ->execute()
        ->fetchAssoc();

      if (!$pattern) {
        return NULL;
      }

      $data = unserialize($pattern['pattern_data']);
      return $this->dataAnonymizer->previewAnonymization($data);
    }
    catch (\Exception $e) {
      $this->loggerFactory->error('Failed to preview pattern sharing: @error', [
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }
}
